@@ -1,5 +1,6 @@
 from sqlalchemy_traversal.interfaces    import ISABase
 from sqlalchemy_traversal.interfaces    import ISASession
+from sqlalchemy_traversal.interfaces    import ISaver
 
 from datetime                           import datetime
 from datetime                           import date
@@ -7,6 +8,32 @@ from datetime                           import time
 
 from sqlalchemy.orm                     import class_mapper
 
+import colander
+import venusian
+
+def format_colander_errors(e):
+    """
+    This formats our colander errors in a nice format
+    for rendering errors
+    """
+    values = []
+
+    for child in e.children:
+        msg = child.msg
+
+        if msg == None:
+            msg = e.asdict()
+
+        error = {
+            'name': child.node.title
+            , 'id': child.node.name
+            , 'error': msg
+        }
+
+        values.append(error)
+
+    return values
+
 def get_session(request):
     """
     This is a utility to allow you to extract the session from the pyramid
@@ -178,6 +205,9 @@ class ModelCollection(TraversalBase):
                 # are looking for?
                 if data == compare_value:
                     obj.__parent__ = self.__parent__
+                    if hasattr(self, '_request'):
+                        obj._request = self._request
+
                     return obj
 
             raise KeyError
@@ -197,6 +227,22 @@ class TraversalMixin(JsonSerializableMixin):
     """
     _traversal_lookup_key = 'id'
 
+    def _recurse_get_traversal_root(self, obj):
+        if hasattr(obj, 'get_class'):
+            return obj
+        else:
+            return self._recurse_get_traversal_root(obj.__parent__)
+
+    def _get_class(self, name):
+        """
+        This is a recursive function that will search every parent
+        until we get to the TraversalRoot so that we can get the
+        class name
+        """
+        root = self._recurse_get_traversal_root(self)
+
+        return root.get_class(name)
+
     def __getitem__(self, attribute):
         """
         This is where the traversal magic happens for a specific model,
@@ -211,28 +257,21 @@ class TraversalMixin(JsonSerializableMixin):
         if hasattr(self, '_request'):
             #TODO: Lets make this less of a hack
             # POST means "create", so we are always looking for a class
-            if self._request.method == 'POST':
+            if self._request.method == 'POST' and \
+                    self._request.path.endswith(attribute):
                 # get an sa mapper class to look up properties on
                 # we need to find out which class the attribute is
                 # attached to and return that
                 mapper = class_mapper(self.__class__, compile=False)
                 rel_prop = mapper.get_property(attribute)
                 name = rel_prop.target.name
-                cls = self.__parent__.get_class(name)()
+                cls = self._get_class(name)()
                 cls.__parent__ = self
+                cls._request = self._request
 
                 return cls
 
-        try:
-            obj = getattr(self, attribute)
-        except AttributeError as e:
-            #TODO: do we need to do anything here?
-            # the object doesn't have the attribte, maybe we want a class?
-            # return from the parent
-            #cls = self.__parent__.get(k)()
-            #cls.__parent__ = self
-            #return cls
-            pass
+        obj = getattr(self, attribute)
 
         # The model had the specific attribute, so we just need to figure out
         # if we are returning a collection or a single instance
@@ -244,6 +283,10 @@ class TraversalMixin(JsonSerializableMixin):
                     iter(obj)
                     col = ModelCollection(obj)
                     col.__parent__ = self
+
+                    if hasattr(self, '_request'):
+                        col._request = self._request
+
                     return col
             except TypeError as e:
                 pass
@@ -253,6 +296,58 @@ class TraversalMixin(JsonSerializableMixin):
         # throws a 404
         raise KeyError
 
+class register_save(object):
+    """
+    This is a decorator that matches a Model class with a colander Schema
+
+    If an API is called with a POST or PUT it will first try to run validation
+    against the schema before doing anything
+    """
+    def __init__(self, cls, schema):
+        self.cls = cls
+        self.schema = schema
+
+    def register(self, scanner, name, wrapped):
+        def save(request):
+            session = get_session(request)
+
+            if request.is_xhr:
+                post_items = request.json.items()
+            else:
+                post_items = request.POST.items()
+
+            schema = self.schema()
+            schema = schema.bind(request=request)
+
+            try:
+                data = schema.deserialize(post_items)
+            except colander.Invalid as e:
+                error_dict = {
+                    'has_errors': True,
+                    'errors': format_colander_errors(e)
+                }
+
+                return dict(error_dict.items() + post_items)
+
+            # cleaned dictionary data from the save function
+            result = wrapped(request, data)
+
+            for key, value in result.iteritems():
+                setattr(request.context, key, value)
+
+            session.add(request.context)
+            session.flush()
+
+            return request.context
+
+        registry = scanner.config.registry
+        registry.registerAdapter(save, (self.cls, ), ISaver)
+
+    def __call__(self, wrapped):
+        venusian.attach(wrapped, self.register)
+
+        return wrapped
+
 def includeme(config):
     config.scan('sqlalchemy_traversal')
     config.include('sqlalchemy_traversal.routes')
